package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.io.FileUtil;
import com.alibaba.excel.EasyExcelFactory;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import pps.cloud.system.service.SystemAreaService;
import pps.cloud.system.service.data.sys_area.QuerySysAreaInput;
import pps.cloud.system.service.data.sys_area.QuerySysAreaOutput;
import pps.core.base.service.data.base_excel.ExcelPhotovoltaicPlantTemplate;
import pps.core.base.utils.EasyExcelUtil;
import pps.core.common.excel.CascadeWriteHandler;
import pps.core.common.excel.ExcelStyleTool;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XStorageTool;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XListResult;
import xstartup.feature.api.annotation.XApiGet;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * excel导入导出
 *
 * @author ZWT
 * @date 2024/05/07 09:49
 */
@XText("Excel")
@XService
public class ExcelService {

    /**
     * 光伏电站模板导出
     *
     * @param context 上下文
     * @return {@link XFileResult}
     */
    @XText("光伏电站模板导出")
    @XApiGet(anonymous = true)
    public XFileResult templatePhotovoltaicPlant(XContext context) {
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/光伏电站模板.xlsx");
        File file = FileUtil.touch(fileSavePath);
        //查地区列表
        SystemAreaService bean = context.getBean(SystemAreaService.class);
        XListResult<QuerySysAreaOutput> result = bean.getProvinceAndCity(context, QuerySysAreaInput.builder()
                .provinceList(CollUtil.toList("陕西", "甘肃"))
                .build());
        result.throwIfFail();
        Map<String, List<QuerySysAreaOutput>> collect = result.getResult().stream()
                .collect(Collectors.groupingBy(QuerySysAreaOutput::getProvince));
        //整理数据，放入一个Map中，mapkey存放父地点，value 存放该地点下的子区域
        Map<String, List<String>> siteMap = new HashMap<>(collect.size());
        List<String> provNameList = new ArrayList<>();
        for (String province : collect.keySet()) {
            provNameList.add(province);
            siteMap.put(province, collect.get(province).stream()
                    .map(QuerySysAreaOutput::getCity)
                    .collect(Collectors.toList())
            );
        }
        //生成模板
        ExcelWriter excelWriter = EasyExcelFactory.write(file)
                .registerWriteHandler(ExcelStyleTool.getStyleStrategy())
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
                .registerWriteHandler(new CascadeWriteHandler(provNameList, siteMap))
                .autoCloseStream(Boolean.TRUE)
                .build();
        WriteSheet writeSheet = EasyExcelUtil.writeSelectedSheet(ExcelPhotovoltaicPlantTemplate.class, 0, "光伏电站");
        excelWriter.write(new ArrayList<String>(), writeSheet);
        excelWriter.finish();
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }
}
